# -*- coding:utf-8 -*-
from __future__ import print_function,division
import numpy as np
from keras import optimizers,losses
from keras.models import Sequential
from keras.preprocessing.image import ImageDataGenerator
from keras.layers import Convolution2D,MaxPooling2D,BatchNormalization
from keras.layers import Dense,Activation,Flatten,Dropout
from keras.utils import to_categorical
'''
Correspondence author: Prof. Feiniu Yuan
Email: yfn@ustc.edu

Please cite the following paper:
Zhijian Yin, Boyang Wan, Feiniu Yuan, Xue Xia, Jinting Shi,
“A deep normalization and convolutional neural network for image smoke detection,”
IEEE Access, vol. 5，pp. 18429 – 18438, 2017.
'''

'''
The datasets for testing are stored under the directory "./data".
The file whose filename ends with a string of "data" is dataset itself, and the file whose filename
ends with "label" is class labels for the corresponding file
'''
# datasets for testing
set1 = np.load(file='../data/set1_48_data.npy')
set1_label = np.load(file='../data/set1_48_label.npy')
set1_label = np.reshape(a=set1_label, newshape=(len(set1_label)))
set1_label_c = to_categorical(y=set1_label,num_classes=2)

set2 = np.load('../data/set2_48_data.npy')
set2_label = np.load('../data/set2_48_label.npy')
set2_label = np.reshape(a=set2_label, newshape=(len(set2_label)))
set2_label_c = to_categorical(y=set2_label,num_classes=2)

val_datagen = ImageDataGenerator()
test_set1 = val_datagen.flow(x=set1,y=set1_label_c,batch_size=200)
test_set2 = val_datagen.flow(x=set2,y=set2_label_c,batch_size=200)

# mertics for testing
def Accuracy_t(y_true,y_pred):
    a=y_true-y_pred
    sample_num=len(y_true)
    acc_num=np.sum(a == 0)
    acc=acc_num/sample_num

    return acc

def DR_t(y_true,y_pred):
    P_num  = np.sum(y_true == 1)
    TP_num = np.sum(np.multiply(y_true,y_pred)==1)
    TPR=TP_num/P_num

    return TPR

def FAR_t(y_true,y_pred):
    N_num = np.sum(y_true == 0)
    a = y_true - y_pred
    FP_num = np.sum(a== -1)

    FPR = FP_num / N_num

    return FPR

def loss(y_true,y_pred):
    l=losses.categorical_crossentropy(y_pred=y_pred,y_true=y_true)
    los=l

    return los


sgd=optimizers.SGD(lr=0.01,momentum=0.9,decay=1e-5)
# build model
model_DNCNN = Sequential()
model_DNCNN.add(Convolution2D(nb_filter=32, nb_col=3, nb_row=3, subsample=(1, 1), border_mode='same', name='CONV1_1', input_shape=(48, 48, 3)))
model_DNCNN.add(BatchNormalization(name='BN1_1'))
model_DNCNN.add(Activation('relu'))
model_DNCNN.add(Convolution2D(nb_filter=32, nb_col=3, nb_row=3, subsample=(1, 1), border_mode='same', name='CONV1_2'))
model_DNCNN.add(BatchNormalization(name='BN1_2'))
model_DNCNN.add(Activation('relu'))
model_DNCNN.add(Convolution2D(nb_filter=64, nb_col=3, nb_row=3, subsample=(1, 1), border_mode='same', name='CONV1_3'))
model_DNCNN.add(BatchNormalization(name='BN1_3'))
model_DNCNN.add(Activation('relu'))
model_DNCNN.add(MaxPooling2D(pool_size=(3, 3), strides=(2, 2), border_mode='same', name='pool1'))

model_DNCNN.add(Convolution2D(nb_filter=64, nb_col=3, nb_row=3, subsample=(1, 1), border_mode='same', name='CONV2_1'))
model_DNCNN.add(BatchNormalization(name='BN2_1'))
model_DNCNN.add(Activation('relu'))
model_DNCNN.add(Convolution2D(nb_filter=64, nb_col=3, nb_row=3, subsample=(1, 1), border_mode='same', name='CONV2_2'))
model_DNCNN.add(BatchNormalization(name='BN2_2'))
model_DNCNN.add(Activation('relu'))
model_DNCNN.add(MaxPooling2D(pool_size=(2, 2), strides=(2, 2), border_mode='valid', name='pool2'))

model_DNCNN.add(Convolution2D(nb_filter=384, nb_col=3, nb_row=3, subsample=(1, 1), border_mode='same', name='CONV3'))
model_DNCNN.add(BatchNormalization(name='BN3'))
model_DNCNN.add(Activation('relu'))
model_DNCNN.add(Convolution2D(nb_filter=384, nb_col=3, nb_row=3, subsample=(1, 1), border_mode='same', name='CONV4'))
model_DNCNN.add(BatchNormalization(name='BN4'))
model_DNCNN.add(Activation('relu'))
model_DNCNN.add(Convolution2D(nb_filter=256, nb_col=3, nb_row=3, subsample=(1, 1), border_mode='same', name='CONV5'))
model_DNCNN.add(BatchNormalization(name='BN5'))
model_DNCNN.add(Activation('relu'))
model_DNCNN.add(MaxPooling2D(pool_size=(2, 2), strides=(2, 2), border_mode='valid', name='pool3'))

model_DNCNN.add(Flatten())
model_DNCNN.add(Dense(output_dim=2048, activation='relu', name='FC1'))
model_DNCNN.add(Dropout(rate=0.5))
model_DNCNN.add(Dense(output_dim=2048, activation='relu', name='FC2'))
model_DNCNN.add(Dropout(rate=0.5))
model_DNCNN.add(Dense(output_dim=2, activation='softmax', name='OUT'))
model_DNCNN.compile(optimizer=sgd, loss=loss, metrics=['accuracy'])

#load weights
model_DNCNN.load_weights(filepath='../test_model/trained by original training-set.hdf5')

test1_predict_result=model_DNCNN.predict_classes(x=set1)
test2_predict_result=model_DNCNN.predict_classes(x=set2)

print ('set1_accuracy',Accuracy_t(y_true=set1_label,y_pred=test1_predict_result))
print ('set1_DR',DR_t(y_true=set1_label,y_pred=test1_predict_result))
print ('set1_FAR',FAR_t(y_true=set1_label,y_pred=test1_predict_result))

print ('set2_ accuracy',Accuracy_t(y_true=set2_label,y_pred=test2_predict_result))
print ('set2_DR',DR_t(y_true=set2_label,y_pred=test2_predict_result))
print ('set2_FAR',FAR_t(y_true=set2_label,y_pred=test2_predict_result))

